@@ -1,6 +1,6 @@
 # -*- test-case-name: twisted.words.test.test_xmlstream -*-
 #
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -82,7 +82,7 @@ class XmlStream(protocol.Protocol, utility.EventDispatcher):
 
         Dispatches the L{STREAM_END_EVENT}.
         """
-        self.dispatch(self, STREAM_END_EVENT)
+        self.dispatch(reason, STREAM_END_EVENT)
         self.stream = None
 
     ### --------------------------------------------------------------
